﻿/*	TOP-SURF: a visual words toolkit
	Copyright (C) 2010 LIACS Media Lab, Leiden University,
	                   Bart Thomee (bthomee@liacs.nl),
					   Erwin M. Bakker (erwin@liacs.nl)	and
					   Michael S. Lew (mlew@liacs.nl).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    See http://www.gnu.org/licenses/gpl.html for the full license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	In addition, this work is covered under the Creative Commons
	Attribution license version 3.
    See http://creativecommons.org/licenses/by/3.0/ for the full license.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;

namespace MediaLab.TopSurf
{
	public partial class CompareDescriptorsForm : Form
	{
		TOPSURF_GUIDATA m_data1;
		public TOPSURF_GUIDATA Data1 { get { return m_data1; } }
		List<TOPSURF_GUIDATA> m_data2;
		public List<TOPSURF_GUIDATA> Data2 { get { return m_data2; } }
		public TOPSURF_SIMILARITY Similarity
		{
			get
			{
				if (cosineButton.Checked)
					return TOPSURF_SIMILARITY.TOPSURF_COSINE;
				else
					return TOPSURF_SIMILARITY.TOPSURF_ABSOLUTE;
			}
		}

		public CompareDescriptorsForm(List<TOPSURF_GUIDATA> data)
		{
			InitializeComponent();
			// fill the boxes with data
			if (data == null || data.Count <= 1)
				throw new ArgumentException();
			foreach (TOPSURF_GUIDATA td in data)
				data1Box.Items.Add(td);
			// select the first item
			// Note: this causes the data1Box_SelectedIndexChanged
			//       function to be called, where we fill the second
			//       box with items
			data1Box.SelectedIndex = 0;
			// prepare data list
			m_data2 = new List<TOPSURF_GUIDATA>();
			// select cosine distance by default
			cosineButton.Checked = true;
		}

		private void CompareDescriptorsForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (this.DialogResult == DialogResult.OK)
			{
				// add the main descriptor for comparison
				m_data1 = (TOPSURF_GUIDATA)data1Box.Items[data1Box.SelectedIndex];
				// add the descriptors to compare with
				foreach (object obj in data2Box.SelectedItems)
					m_data2.Add((TOPSURF_GUIDATA)obj);
			}
		}

		private void data1Box_SelectedIndexChanged(object sender, EventArgs e)
		{
			// fill the second box with all items from the
			// first, except the currently selected one
			data2Box.Items.Clear();
			data2Box.Items.AddRange(data1Box.Items);
			data2Box.Items.Remove(data1Box.SelectedItem);
			// select all items
			for (int i = 0; i < data2Box.Items.Count; i++)
				data2Box.SetSelected(i, true);
		}

		private void data2Box_KeyUp(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.A && e.Control)
			{
				// select all items
				for (int i = 0; i < data2Box.Items.Count; i++)
					data2Box.SetSelected(i, true);
			}
		}
	}
}
