﻿/*	TOP-SURF: a visual words toolkit
	Copyright (C) 2010 LIACS Media Lab, Leiden University,
	                   Bart Thomee (bthomee@liacs.nl),
					   Erwin M. Bakker (erwin@liacs.nl)	and
					   Michael S. Lew (mlew@liacs.nl).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    See http://www.gnu.org/licenses/gpl.html for the full license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	In addition, this work is covered under the Creative Commons
	Attribution license version 3.
    See http://creativecommons.org/licenses/by/3.0/ for the full license.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace MediaLab.TopSurf
{
	public partial class LoadDictionaryForm : Form
	{
		public string Directory { get { return directoryBox.Text; } }

		public LoadDictionaryForm()
		{
			InitializeComponent();
			// set the initial directory
			directoryBox.Text = System.IO.Directory.GetCurrentDirectory();
		}

		private void browseButton_Click(object sender, EventArgs e)
		{
			using (FolderBrowserDialog fbd = new FolderBrowserDialog())
			{
				fbd.Description = "Please choose directory to load the dictionary from.";
				fbd.SelectedPath = directoryBox.Text;
				fbd.ShowNewFolderButton = false;
				if (fbd.ShowDialog() != DialogResult.OK)
					return;
				directoryBox.Text = fbd.SelectedPath;
			}
		}

		private void LoadDictionaryForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (this.DialogResult == DialogResult.OK)
			{
				// check if all dictionary files are inside the directory
				string dictionaryPath = Path.Combine(directoryBox.Text, "dictionary.txt");
				string idfPath = Path.Combine(directoryBox.Text, "idf.dat");
				string kdtreePath = Path.Combine(directoryBox.Text, "kdtree.dat");
				string visualwordsPath = Path.Combine(directoryBox.Text, "visualwords.dat");
				if (!File.Exists(dictionaryPath) || !File.Exists(idfPath) ||
					!File.Exists(kdtreePath) || !File.Exists(visualwordsPath))
				{
					MessageBox.Show("Could not find a TOP-SURF dictionary in the selected directory", "Dictionary not found", MessageBoxButtons.OK, MessageBoxIcon.Error);
					e.Cancel = true;
				}
			}
		}

		private void directoryBox_Enter(object sender, EventArgs e)
		{
			directoryBox.SelectAll();
		}
	}
}
