﻿/*	TOP-SURF: a visual words toolkit
	Copyright (C) 2010 LIACS Media Lab, Leiden University,
	                   Bart Thomee (bthomee@liacs.nl),
					   Erwin M. Bakker (erwin@liacs.nl)	and
					   Michael S. Lew (mlew@liacs.nl).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    See http://www.gnu.org/licenses/gpl.html for the full license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	In addition, this work is covered under the Creative Commons
	Attribution license version 3.
    See http://creativecommons.org/licenses/by/3.0/ for the full license.
*/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace MediaLab.TopSurf
{
	// log to a list box
	public class Logger : ListBox
	{
		// a log entry
		private class LogEntry
		{
			// the message
			public string message;
			// the type of message
			public enum LOG_TYPE
			{
				LOG_NORMAL,
				LOG_ERROR
			};
			public LOG_TYPE type;
			// construct a message
			public LogEntry(string _message, LOG_TYPE _type)
			{
				message = _message;
				type = _type;
			}
			// print out the message
			public override string ToString()
			{
				return message;
			}
		}

		public Logger()
		{
			this.DrawMode = DrawMode.OwnerDrawFixed;
		}

		protected override void OnDrawItem(DrawItemEventArgs e)
		{
			if (e != null && e.Index >= 0 && e.Index < this.Items.Count)
			{
				// draw the background
				e.DrawBackground();
				// draw error messages in red and normal messages in the default color
				Brush textBrush;				
				LogEntry logEntry = (LogEntry)this.Items[e.Index];
				if (logEntry.type == LogEntry.LOG_TYPE.LOG_ERROR)
					textBrush = Brushes.Red;
				else
					textBrush = SystemBrushes.ControlText;
				e.Graphics.DrawString(logEntry.message, e.Font, textBrush, e.Bounds);
				// if the ListBox has focus, draw a focus rectangle around the selected item
				e.DrawFocusRectangle();
			}
		}

		public void WriteNormal(string message)
		{
			ControlHelper.AsyncInvoke(this, delegate()
			{
				if (this.Items.Count == 1000)
					this.Items.RemoveAt(0);
				this.Items.Add(new LogEntry(message, LogEntry.LOG_TYPE.LOG_NORMAL));
				this.SelectedIndex = this.Items.Count - 1;
				this.Refresh();
			});
		}

		public void WriteError(string message)
		{
			ControlHelper.AsyncInvoke(this, delegate()
			{
				if (this.Items.Count == 1000)
					this.Items.RemoveAt(0);
				this.Items.Add(new LogEntry(message, LogEntry.LOG_TYPE.LOG_ERROR));
				this.SelectedIndex = this.Items.Count - 1;
				this.Refresh();
			});
		}

		public void Redraw()
		{
			ControlHelper.AsyncInvoke(this, delegate()
			{
				this.Refresh();
			});
		}
	}
}
