﻿/*	TOP-SURF: a visual words toolkit
	Copyright (C) 2010 LIACS Media Lab, Leiden University,
	                   Bart Thomee (bthomee@liacs.nl),
					   Erwin M. Bakker (erwin@liacs.nl)	and
					   Michael S. Lew (mlew@liacs.nl).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    See http://www.gnu.org/licenses/gpl.html for the full license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	In addition, this work is covered under the Creative Commons
	Attribution license version 3.
    See http://creativecommons.org/licenses/by/3.0/ for the full license.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace MediaLab.TopSurf
{
	public partial class SaveDescriptorsForm : Form
	{
		List<TOPSURF_GUIDATA> m_data;
		public List<TOPSURF_GUIDATA> Data { get { return m_data; } }
		string m_folder;
		public string Folder { get { return m_folder; } }

		public SaveDescriptorsForm(List<TOPSURF_GUIDATA> data)
		{
			InitializeComponent();
			// fill the list with data
			if (data == null || data.Count == 0)
				throw new ArgumentException();
			foreach (TOPSURF_GUIDATA td in data)
				dataBox.Items.Add(td);
			// select all items
			for (int i = 0; i < dataBox.Items.Count; i++)
				dataBox.SetSelected(i, true);
			// prepare data list
			m_data = new List<TOPSURF_GUIDATA>();
			// select same folder by default
			sameButton.Checked = true;
		}

		private void SaveDescriptorsForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (this.DialogResult == DialogResult.OK)
			{
				// check where the user wants to save the descriptors
				if (sameButton.Checked)
					m_folder = null;
				else
				{
					using (FolderBrowserDialog fbd = new FolderBrowserDialog())
					{
						fbd.Description = "Please choose directory where to save the descriptors.";
						fbd.SelectedPath = Directory.GetCurrentDirectory();
						fbd.ShowNewFolderButton = false;
						if (fbd.ShowDialog() != DialogResult.OK)
						{
							e.Cancel = true;
							return;
						}
						m_folder = fbd.SelectedPath;
					}
				}
				// add the descriptors to save
				foreach (object obj in dataBox.SelectedItems)
					m_data.Add((TOPSURF_GUIDATA)obj);
			}
		}

		private void dataBox_KeyUp(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.A && e.Control)
			{
				// select all items
				for (int i = 0; i < dataBox.Items.Count; i++)
					dataBox.SetSelected(i, true);
			}
		}
	}
}
