﻿/*	TOP-SURF: a visual words toolkit
	Copyright (C) 2010 LIACS Media Lab, Leiden University,
	                   Bart Thomee (bthomee@liacs.nl),
					   Erwin M. Bakker (erwin@liacs.nl)	and
					   Michael S. Lew (mlew@liacs.nl).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    See http://www.gnu.org/licenses/gpl.html for the full license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	In addition, this work is covered under the Creative Commons
	Attribution license version 3.
    See http://creativecommons.org/licenses/by/3.0/ for the full license.
*/

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;

namespace MediaLab.TopSurf
{
	public class TOPSURF_GUIDATA : IDisposable
	{
		string m_name;
		public string Name { get { return m_name; } }
		string m_path;
		public string Path { get { return m_path; } }
		Bitmap m_image;
		public Bitmap Image { get { return m_image; } }
		Bitmap m_visualize;
		public Bitmap Visualize { get { return m_visualize; } }
		TOPSURF_CXX m_descriptor;
		public TOPSURF_CXX Descriptor { get { return m_descriptor; } }

		public override string ToString()
		{
			return m_name;
		}

		public TOPSURF_GUIDATA(string path, Bitmap image, Bitmap visualize, TOPSURF_CXX descriptor)
		{
			m_name = System.IO.Path.GetFileNameWithoutExtension(path);
			m_path = path;
			m_image = image;
			m_visualize = visualize;
			m_descriptor = descriptor;
		}

		~TOPSURF_GUIDATA()
		{
			// finalizer calls Dispose(false)
			Dispose(false);
		}

		public void Dispose()
		{
			// Dispose() calls Dispose(true)
			Dispose(true);
			GC.SuppressFinalize(this);
		}
		
		protected virtual void Dispose(bool disposing)
		{
			// release unmanaged data
			if (disposing)
			{
				if (m_image != null)
				{
					m_image.Dispose();
					m_image = null;
				}
				if (m_visualize != null)
				{
					m_visualize.Dispose();
					m_visualize = null;
				}
			}
		}
	}
}
