﻿/*	TOP-SURF: a visual words toolkit
	Copyright (C) 2010 LIACS Media Lab, Leiden University,
	                   Bart Thomee (bthomee@liacs.nl),
					   Erwin M. Bakker (erwin@liacs.nl)	and
					   Michael S. Lew (mlew@liacs.nl).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    See http://www.gnu.org/licenses/gpl.html for the full license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

	In addition, this work is covered under the Creative Commons
	Attribution license version 3.
    See http://creativecommons.org/licenses/by/3.0/ for the full license.
*/

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Windows.Forms;

namespace MediaLab.TopSurf
{
	public class TOPSURF_FORM : IDisposable
	{
		List<Form> m_forms;
		//public List<ImageForm> Forms { get { return m_forms; } }
		public int Count { get { return m_forms.Count; } }

		public TOPSURF_FORM()
		{
			m_forms = new List<Form>();
		}

		public void Add(Form form)
		{
			form.FormClosed += new System.Windows.Forms.FormClosedEventHandler(FormClosed);
			m_forms.Add(form);
		}

		public void CloseAll()
		{
			// Note: because the close action causes the form list
			//       to be modified by removing the form, we cannot
			//       use a for-loop
			while (m_forms.Count > 0)
				m_forms[0].Close();
		}

		void FormClosed(object sender, System.Windows.Forms.FormClosedEventArgs e)
		{
			Form form = (Form)sender;
			m_forms.RemoveAll(x => x == form);
		}

		~TOPSURF_FORM()
		{
			// finalizer calls Dispose(false)
			Dispose(false);
		}

		public void Dispose()
		{
			// Dispose() calls Dispose(true)
			Dispose(true);
			GC.SuppressFinalize(this);
		}
		
		protected virtual void Dispose(bool disposing)
		{
			// release unmanaged data
			if (disposing)
				CloseAll();
		}
	}
}
